 /**
 * PS Move API - An interface for the PS Move Motion Controller
 * Copyright (c) 2014 Jonathan Whiting <jonathan@ignika.com>
 * Copyright (c) 2016 Thomas Perl <m@thp.io>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 **/

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

#include "psmoveapi.h"

// Private header for psmove_port_sleep_ms()
#include "../../src/psmove_port.h"

namespace {

struct ColorHandler : public psmoveapi::Handler {
    ColorHandler()
        : quit(false)
        , connected(0)
    {
    }

    virtual void connect(Controller *controller) {
        printf("Connected: %s\n", controller->serial);
        connected++;
    }

    virtual void disconnect(Controller *controller) {
        printf("Disconnected: %s\n", controller->serial);
        connected--;
    }

    virtual void update(Controller *controller) {
        RGB &out = controller->color;
        switch (controller->battery) {
            case Batt_MIN:           out.r = 1.f;  out.g = 0.f;  out.b = 0.f; break;
            case Batt_20Percent:     out.r = 1.f;  out.g = 0.5f; out.b = 0.f; break;
            case Batt_40Percent:     out.r = 1.f;  out.g = 1.f;  out.b = 0.f; break;
            case Batt_60Percent:     out.r = 0.6f; out.g = 1.f;  out.b = 0.f; break;
            case Batt_80Percent:     out.r = 0.3f; out.g = 1.f;  out.b = 0.f; break;
            case Batt_MAX:           out.r = 0.f;  out.g = 1.f;  out.b = 0.f; break;
            case Batt_CHARGING:      out.r = 0.5f; out.g = 0.f;  out.b = 1.f; break;
            case Batt_CHARGING_DONE: out.r = 1.f;  out.g = 0.f;  out.b = 1.f; break;
            default:                 out.r = 0.f;  out.g = 0.f;  out.b = 0.f; break;
        }

        if ((controller->buttons & Btn_PS) != 0) {
            quit = true;
        }
    }

    bool quit;
    int connected;
};

}; // end anonymous namespace


int
main(int argc, char *argv[])
{
    ColorHandler handler;

    psmoveapi::PSMoveAPI api(&handler);

    while (!handler.quit) {
        psmove_port_sleep_ms(100);
        api.update();

        if (handler.connected == 0) {
            break;
        }
    }

    return 0;
}
